
/*
 *	dinv.h
 */

#ifndef _DINV_H_
#define _DINV_H_

#include "gba.h"

/*
 *	Game
 */

#define PG_STAGECLEAR	0x0001
#define PG_FAILURE	0x0002
#define PG_GAMEOVER	0x0004

/*
 *	Timer
 */

#define INFINITE	((u32)0xffffffff)

#define TIMER_INTMS	10

#define CLOCK_FREQ	((16*1024*1024) / 64)
#define TIMER_INTERVAL	((CLOCK_FREQ * TIMER_INTMS) / 1000)
#define TIMER_COUNTER	(65536 - TIMER_INTERVAL)

extern vu32	ClockTick;

/*
 *	Key
 */

#define KEY_REPEAT	0x4000

/*
 *	External function definitions
 */

/* font8.c */
void InitFont8(int x, int y);
void Put8Char(u8 c, int x, int y);
void Put8Str(u8 *str, int x, int y);

/* game.c */
void InitGame();
void StartGame();
void InitStage();
void StartStage();
void NextStage();
void GameOver();
int PlayGame();

/* led.c */
void InitLED(int x, int y);
void PutLED(int offset, int ledno);
void PutLEDStr(int offset, u8 *s);
void ClearLED();

/* sprintf.c */
void _sprintf(u8 *bp, u8 *fmt, ...);

/* sram.c */
u32 LoadHighScore();
void SaveHighScore(u32 HighScore);
void InitSRAM();

/* subr.c */
void Srand(u32 seed);
u32 Rand();
int GetKey();
void FlashTextAndWait(int x, int y, u8 *str, u16 keymask, u32 timeout);

/* timer.c */
void InitTimer();
void Wait(u32 msec);
u16 KeyWait(u16 keyptn, u32 msec);

#endif // _DINV_H_

