/*
 *	game.c
 */
#include "dinv.h"

#define LEDNO_0		'0'
#define LEDNO_UFO	('9' + 1)

#define GK_AIM		(KEY_UP | KEY_LEFT | KEY_B)
#define GK_FIRE		(KEY_DOWN | KEY_RIGHT | KEY_A)

#define MAX_PART	2
#define MAX_STAGE	9

#define MIN_SPEED	600
#define MAX_SPEED	2000
#define DIF_SPEED	((MAX_SPEED - MIN_SPEED) / (MAX_STAGE - 1))
#define STAGE_SPEED(st)	(MAX_SPEED - (st) * DIF_SPEED)

static const int	StageSpeed[MAX_STAGE] = {
	STAGE_SPEED(0),
	STAGE_SPEED(1),
	STAGE_SPEED(2),
	STAGE_SPEED(3),
	STAGE_SPEED(4),
	STAGE_SPEED(5),
	STAGE_SPEED(6),
	STAGE_SPEED(7),
	STAGE_SPEED(8),
};

static const int	PartStartPos[MAX_PART] = { 7, 6 };

static int	PlayerLeft;
static int	Part;
static int	Stage;

static int	nInvader;
static int	nRemain;
static int	nShot;

static int	CurAim;
static int	Sum;
static BOOL	fUFO;

static int	Scrn[8];

static u32	HighScore;
static u32	Score;

/*
 *	InitGame
 */
void InitGame()
{
	HighScore = LoadHighScore();
	Score = 0;
}

/*
 *	UpdateHighScore
 */
static void UpdateHighScore()
{
	if (Score > HighScore) {
		HighScore = Score;
		SaveHighScore(HighScore);
	}
}

/*
 *	PutScore
 */
static void PutScore()
{
	u8		buf[16];

	ClearLED();
	_sprintf(buf, "%d-%06d", Stage + 1, Score);
	PutLEDStr(0, buf);
}

/*
 *	StartGame
 */
void StartGame()
{
	Score = 0;
	PlayerLeft = 2;
	Part = 0;
	Stage = 0;
	Srand(HighScore + ClockTick);
}

/*
 *	InitStage
 */
void InitStage()
{
	nInvader = 16;
}

/*
 *	StartStage
 */
void StartStage()
{
	u8		buf[16];

	nShot = 30;

	ClearLED();

	_sprintf(buf, "%d-%06d", Stage + 1, HighScore);
	PutLEDStr(0, buf);
	KeyWait(GK_AIM, 3000);

	_sprintf(buf, "   %02d-%02d", nInvader, nShot);
	PutLEDStr(0, buf);
	KeyWait(GK_AIM, 3000);
}

/*
 *	NextStage
 */
void NextStage()
{
	UpdateHighScore();
	PutScore();
	FlashTextAndWait(3, 0, "Stage Clear!", GK_AIM, 2400);

	if (++Stage >= MAX_STAGE) {
		Stage = 0;
		if (++Part >= MAX_PART)
			Part = 0;
	}
}

/*
 *	GameOver
 */
void GameOver()
{
	PutScore();
	FlashTextAndWait(3, 0, "Game Over!", 0, 3000);
}

static int	CurPos;

/*
 *	PlayGame
 */
int PlayGame()
{
	int		i, j;
	int		r;
	int		sp;
	u16		key;
	u32		tmo;

	ClearLED();

	CurAim = LEDNO_0;
	PutLED(0, CurAim);
	PutLED(1, PlayerLeft + 0x10);

	Sum = 0;
	nRemain = nInvader;
	fUFO = FALSE;
	sp = PartStartPos[Part];

	for (i = 2; i <= sp; i++)
		Scrn[i] = 0;

	for (;;) {
		if (Scrn[2]) {
			UpdateHighScore();

			if (--PlayerLeft < 0)
				return PG_GAMEOVER;
			FlashTextAndWait(5, 0, "Failure!", 0, 2000);

			nInvader = nRemain;

			return PG_FAILURE;
		}
		for (i = 2; i < sp; i++) {
			Scrn[i] = Scrn[i + 1];
			PutLED(i, Scrn[i]);
		}
		if (nInvader > 0) {
			if (fUFO) {
				Scrn[i] = LEDNO_UFO;
				fUFO = FALSE;
				nRemain++;
			} else {
				Scrn[i] = (Rand() % 10) + LEDNO_0;
				nInvader--;
			}
		} else
			Scrn[i] = 0;
		PutLED(i, Scrn[i]);

		tmo = ClockTick + StageSpeed[Stage];
		while (ClockTick < tmo) {
			key = GetKey();
			if (key & GK_AIM) {
				if (++CurAim > LEDNO_UFO)
					CurAim = LEDNO_0;
				PutLED(0, CurAim);
			} else if (key & GK_FIRE) {
				if (--nShot < 0) {
					UpdateHighScore();
					return PG_GAMEOVER;
				}

				for (i = 2; i <= sp; i++)
					if (Scrn[i] == CurAim)
						break;
				if (i > sp)
					continue;	// Miss

				if (Scrn[i] == LEDNO_UFO) {
					Score += 300;

					ClearLED();
					Wait(300);

					PutLED(0, CurAim);
					PutLED(1, PlayerLeft + 0x10);
					for (j = 2; j <= sp; j++)
						PutLED(j, Scrn[j]);
				} else {
					Score += (i - 1) * 10 * (Part + 1);

					Sum += CurAim - LEDNO_0;
					if ((Sum % 10) == 0)
						fUFO = TRUE;
				}
				if (--nRemain <= 0)
					return PG_STAGECLEAR;

				Scrn[i] = 0;
				for (j = i; j > 2; j--) {
					Scrn[j] = Scrn[j - 1];
					PutLED(j, Scrn[j]);
				}
				Scrn[j] = 0;
				PutLED(j, Scrn[j]);
			} else if (key & KEY_START) {
				FlashTextAndWait(7, 0, "Pause",
					KEY_START, INFINITE);
			}
		}
	}
}

